@@ -21,13 +21,17 @@ __author__ = 'Victor Olaya'
 __date__ = 'January 2016'
 __copyright__ = '(C) 2016, Victor Olaya'
 # This will get replaced with a git SHA1 when you do a git archive
-__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
+__revision__ = '93e761dd2396af21db9bb93e26d7cba680c906ca'
 
 import os
 import codecs
-import yaml
+import warnings
+with warnings.catch_warnings():
+    warnings.filterwarnings("ignore", category=DeprecationWarning)
+    import yaml
+
 from qgis.core import QgsSettings, Qgis
-from qgis.PyQt.QtCore import QLocale
+from qgis.PyQt.QtCore import QLocale, QCoreApplication
 
 
 def loadShortHelp():
@@ -37,7 +41,13 @@ def loadShortHelp():
         if f.endswith("yaml"):
             filename = os.path.join(path, f)
             with codecs.open(filename, encoding='utf-8') as stream:
-                h.update(yaml.load(stream))
+                with warnings.catch_warnings():
+                    warnings.filterwarnings("ignore", category=DeprecationWarning)
+                    for k, v in yaml.load(stream).items():
+                        if v is None:
+                            continue
+                        h[k] = QCoreApplication.translate("{}Algorithm".format(f[:-5].upper()), v)
+
     version = ".".join(Qgis.QGIS_VERSION.split(".")[0:2])
     overrideLocale = QgsSettings().value('locale/overrideFlag', False, bool)
     if not overrideLocale:
@@ -51,7 +61,9 @@ def loadShortHelp():
             return s.replace("{qgisdocs}", "https://docs.qgis.org/%s/%s/docs" % (version, locale))
         else:
             return None
+
     h = {k: replace(v) for k, v in list(h.items())}
+
     return h
 
 
