@@ -23,7 +23,7 @@ __copyright__ = '(C) 2017, Nyall Dawson'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '9b176802e5512bc310765d8ebe4e44b5f4663453'
+__revision__ = '93e761dd2396af21db9bb93e26d7cba680c906ca'
 
 import os
 
@@ -306,7 +306,7 @@ class SpatialJoinSummary(QgisAlgorithm):
 
                 join_attributes = []
                 for a in join_field_indexes:
-                    join_attributes.append(test_feat.attributes()[a])
+                    join_attributes.append(test_feat[a])
 
                 if engine is None:
                     engine = QgsGeometry.createGeometryEngine(f.geometry().constGet())
